import yaml, asyncio
import gradio as gr
from loguru import logger
from WebUI.benchmarks.constant import (
    RAG_CONFIG_PATH,
)

RAG_REMINDER = "Please set the Config Params ——————>\n\n\nAnd press the save button below."

def read_config_file(path: str) -> str:
    """读取指定路径的配置文件内容"""
    if not path.strip():
        return ""
    try:
        with open(path, "r", encoding="utf-8") as f:
            return f.read()
    except FileNotFoundError:
        logger.warning(f"配置文件未找到: {path}")
        return "文件未找到或路径错误"
    except Exception as e:
        logger.error(f"读取配置文件失败: {str(e)}")
        return f"读取失败: {str(e)}"
    

def parse_rag_config(content: str) -> dict:
    """解析RAG配置内容"""
    try:
        return yaml.safe_load(content) or {}
    except Exception as e:
        logger.error(f"解析RAG配置失败: {str(e)}")
        return {"error": f"配置解析失败: {str(e)}"}

def save_rag_config(*args) -> str:
    config_data = {}
    try:
        keys = args[:len(args)//2]
        values = args[len(args)//2:]
        
        for key, value in zip(keys, values):
            try:
                parsed_value = yaml.safe_load(value)
            except:
                parsed_value = value
            
            save_key = key if "_" in key else "_".join(key.split()).lower()
                

            config_data[save_key] = parsed_value
        
        if not isinstance(config_data, dict):
            raise ValueError("Configuration format must be key-value pairs")
        
            
        try:
            with open(RAG_CONFIG_PATH, "w", encoding="utf-8") as f:
                yaml.dump(config_data, f, allow_unicode=True, sort_keys=False)
            return "✅ RAG Configuration save successfully!"
        except Exception as e:
            return f"❌ Save Failed: {str(e)}"
        
    except Exception as e:
        logger.error(f"Save Configuration Failed: {str(e)}")
        return f"❌ Error: {str(e)}"
    
async def save_rag_config_ui(*args):
    ret = save_rag_config(*args)
    yield gr.update(visible = True, value = ret)
    await asyncio.sleep(5)
    yield gr.update(visible = False, value = RAG_REMINDER)



initial_rag_content = read_config_file(RAG_CONFIG_PATH)

RAG_CONFIG = parse_rag_config(initial_rag_content)