import os, time, asyncio
import gradio as gr
from loguru import logger
from filelock import FileLock
from WebUI.benchmarks.constant import (
    CATEGORY_BENCHMARKS_DICT,
    BENCHMARK_TASKS_DICT,
    CFG_DEFAULT,
    TASK_TO_BENCH,
    BENCHMARK_NAMES,
    BENCHMARK_REMINDER,
    ROOT_PATH,
    CURRENT_CONSOLE_INDEX_PATH,
    TMP_PATH
)

from WebUI.utils.evaluate import remove_progress_bar

ALREADY_LOCK = FileLock(f"{TMP_PATH}/alreay.lock")


def child_update_yamlbox(subchoice):
    choice = TASK_TO_BENCH[subchoice]
    if choice == "/":
        return [gr.update(label = "" , visible = True,value = BENCHMARK_REMINDER), gr.update(value = CFG_DEFAULT)]
    label = subchoice + ".yaml"
    CONFIG_ROOT = os.path.join(ROOT_PATH,"benchmarks")
    path = None
    for dir in os.listdir(CONFIG_ROOT):
        dir_path = os.path.join(CONFIG_ROOT, dir)
        if dir[0].upper()!=dir[0] or os.path.isfile(dir_path):
            continue
        subfile = os.listdir(dir_path)
        if choice not in subfile:continue
        path = os.path.join(dir_path, choice, "configs", label)
        break

    with open(path, "r") as f:
        value = f.read()
    return[gr.update(label = label,
                     value = value,
                     visible = False
                     ), gr.update(value = "." + path.split(os.path.abspath(ROOT_PATH))[-1])]

def update_task(parent_choice):
    new_choices = BENCHMARK_TASKS_DICT[parent_choice]
    return [gr.update(choices=new_choices, value = new_choices[0])]

def update_pages(choice):
    return [gr.update(visible=(choice == page)) for page in ["/"] + BENCHMARK_NAMES]

def update_benchmark(choice):
    new_choices = CATEGORY_BENCHMARKS_DICT[choice]
    return [gr.update(choices = new_choices, value = new_choices[0])] 


def update_model_savetag(choice, cfg_path):
    model = os.path.basename(choice.rstrip("/"))
    cfg = ".".join(os.path.basename(cfg_path.rstrip("/")).split('.')[:-1])
    return f"{model}_{cfg}"

def update_config_savetag(choice, model_path):

    model = os.path.basename(model_path.rstrip().rstrip("/"))
    cfg = ".".join(os.path.basename(choice.rstrip("/")).split('.')[:-1])
    return f"{model}_{cfg}"


def update_CATEGORY(choice):
    new_choices = CATEGORY_BENCHMARKS_DICT[choice]
    return gr.update(choices = new_choices, value = new_choices[-1] if choice == "/" else new_choices[0])
    return [gr.update(choices = new_choices, value = new_choices[0])] + update_BENCHMARK(new_choices[0])

def update_BENCHMARK(choice):
    new_choices = BENCHMARK_TASKS_DICT[choice]
    return [gr.update(choices=new_choices, value = new_choices[0])] + [gr.update(visible=(choice == page)) for page in ["/"] + BENCHMARK_NAMES]
    return gr.update(choices=new_choices, value = new_choices[0])
    return [gr.update(choices=new_choices, value = new_choices[0])]# + update_TASK(new_choices[0])

def update_TASK(choice):
    return child_update_yamlbox(choice)
    return [gr.update(visible=(choice == page)) for page in ["/"] + BENCHMARK_NAMES] + child_update_yamlbox(choice)





def already_used():
    if not os.path.exists(f"{TMP_PATH}/already.txt"):
        with open(f"{TMP_PATH}/already.txt", "w") as f:...
        return []
    with open(f"{TMP_PATH}/already.txt", "r") as f:
        already = [k.strip() for k in f.readlines() if k.strip()]
    return already

def choose_latest_folder():
    while not os.path.exists(f"{TMP_PATH}/starting"):
        time.sleep(0.5)

    with ALREADY_LOCK:
        os.remove(f"{TMP_PATH}/starting")
        progresses = [k for k in os.listdir(TMP_PATH) if "progress_bar" in k]
        progresses = list(set(progresses) - set(already_used()))
        # assert len(progresses) == 1, progresses

    if not progresses:progresses = ["progress_bar0"]

    with open(f"{TMP_PATH}/already.txt","a") as f:
        f.write(progresses[0] + "\n")
    
    return f"{TMP_PATH}/{progresses[0]}"


def get_total():
    folder = choose_latest_folder()
    if not os.path.exists(folder):return 0, folder
    while os.path.exists(folder) and "total.txt" not in os.listdir(folder):
        time.sleep(1)
    
    with FileLock(folder.replace("tmp", "tmp/locks")):
        with open(f"{folder}/total.txt", "r") as f:
            total = int(f.readline().strip())

    return total, folder



def progress_bar(total, folder):
    last_position = 0
    current = 0
    try:

        while not os.path.exists(f"{folder}/current.txt"):
            time.sleep(1)

        while True:
            if current + 1 >=total:
                break

            with ALREADY_LOCK:
                with FileLock(folder.replace("tmp", "tmp/locks")):
                    with open(f"{folder}/current.txt", 'r') as f:
                        f.seek(0, 2)
                        if f.tell() == last_position: continue
                        f.seek(last_position, 0)

                        lines = f.readlines()
                        for _ in lines:
                            yield 1
                            
                        current += len(lines)
                        last_position = f.tell()

            time.sleep(0.1)
        with open(f"{folder}/pb_over", "w") as f:
            return

    except Exception as e:
        error_msg = f"An error occurred during execution: {str(e)}"
        logger.error(error_msg)
        yield 1


def update_progress(progress = gr.Progress()):
    total, folder = get_total()
    for _ in progress.tqdm(progress_bar(total, folder), 
                           total = total): 
        pass
        #  progress(1)

    return ""



def server_change(s, *args):
    first =[gr.update(visible=(s == "vllm"))] * 2
    CURRENT_CONSOLE_INDEX = int(open(CURRENT_CONSOLE_INDEX_PATH, "r").read())
    
    return first + [gr.update(visible = (k == CURRENT_CONSOLE_INDEX)) \
             for k in range(len(args))]  + \
              [gr.update(visible = (k == CURRENT_CONSOLE_INDEX) &(s=='vllm')) \
             for k in range(len(args))]*3 + \
                [gr.update(visible = False) if k!=CURRENT_CONSOLE_INDEX \
                 else gr.update(visible = True, lines = 20 - 8*int(s =="vllm"), max_lines = 20 - 8*int(s =="vllm")) \
                 for k in range(len(args))]