# Copyright 2024 GLM-4-9B Model Team @ Zhipu AI
# Copyright 2024 ByteDance and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import torch
import torch.nn.functional as F


def glm_mlp_forward(self, hidden_states):
    def inner_mlp_forward(x):
        # [s, b, 4hp]
        intermediate_parallel = self.dense_h_to_4h(x)
        intermediate_parallel = self.activation_func(intermediate_parallel)
        # [s, b, h]
        output = self.dense_4h_to_h(intermediate_parallel)
        return output

    batch_size, seq_len, hidden_dim = hidden_states.shape
    chunk_size = 32768
    output = torch.empty_like(hidden_states)
    for b in range(batch_size):
        for i in range(0, seq_len, chunk_size):
            output[b : b + 1, i : i + chunk_size] = inner_mlp_forward(
                hidden_states[b : b + 1, i : i + chunk_size]
            )
    return output
