
from openai import OpenAI
import numpy as np
import time,random
from models.rag.bass_class import Base
class openai(Base):
    def __init__(self,config):
        super().__init__(config)
        self.retrieval_methods = "openai"
        self.embedding_model = config["embedding_model"]
        self.client = OpenAI(api_key=config["openai_api"],base_url=config["base_url"])
    def get_embedding(self,texts):
        time.sleep(random.random())
        response = self.client.embeddings.create(input=texts, model=self.embedding_model)
        openai_embeddings = []
        for i in range(len(response.data)):
            openai_embeddings.append(response.data[i].embedding)
        return openai_embeddings
    def get_cosine_similarity(self,vec1, vec2):
        return np.dot(vec1, vec2) / (np.linalg.norm(vec1) * np.linalg.norm(vec2))
    def retrieve_relevant_chunks_for_question(self,context, questions ):
        chunks = self.chunk_text(context)
        context_embeddings = []
        for context in chunks:
            context_embeddings.extend(self.get_embedding(context))
        question_embeddings = self.get_embedding(questions) 
        relevant_chunks = []
        for question_embedding in question_embeddings:
            similarities = [self.get_cosine_similarity(question_embedding, context_embedding) for context_embedding in context_embeddings]
            top_indices = np.argsort(similarities)[-self.num_chunks:]
            top_chunks = [chunks[idx] for idx in top_indices]
            relevant_chunks.extend(top_chunks)
        return relevant_chunks
