

import sys

import yaml
from loguru import logger

logger.remove();logger.add(sys.stdout,colorize=True, format="<level>{message}</level>")
class RagLoader:
    def __init__(self,rag_method,devices,index):
        self.rag_method = rag_method
        self.devices = devices
        self.index = index
    def load(self,rag_config_path):
        with open(rag_config_path,"r") as f:
            config = yaml.safe_load(f)
        if self.rag_method  =="llamaindex":
            from models.rag.llamaindex.llamaindex import llamaindex
            rag = llamaindex(config,self.devices,self.index)
        elif self.rag_method  =="BM25":
            from models.rag.BM25.BM25 import BM25
            rag = BM25(config )
        elif self.rag_method  =="contriever":
            from models.rag.contriever.contriever import contriever
            rag = contriever(config,self.devices,self.index)
        elif self.rag_method  =="openai":
            from models.rag.openai.openai import openai
            rag = openai(config)
        else:
            raise ValueError(f"The specified RAG method {self.rag_method} does not exist.")
        return rag